﻿//////////////////////////////////////////////
// ImageEnvironmentEffect.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Program ;
	class Sampler ;
	class Texture ;
}

/// Includes ---------------------------------

// nkAstraeus
#include "../../../Dll/DllDefines.h"

#include "../Effect.h"

#include "ImageEnvironmentEffectInfo.h"

/// Class ------------------------------------

namespace nkAstraeus
{
	class DLL_ASTRAEUS_EXPORT ImageEnvironmentEffect final : public Effect
	{
		private :

			// Attributes
			// Program
			nkGraphics::Program* _program ;
			ImageEnvironmentEffectInfo _programInfo ;

			// Info
			ImageEnvironmentEffectInfo _info ;

		private :

			// Functions
			// Program
			bool _prepareProgram () ;

			// Shader
			bool _prepareShader () ;

		public :

			// Constructor, destructor
			ImageEnvironmentEffect () ;
			~ImageEnvironmentEffect () ;

			// Getters
			virtual EFFECT_TYPE getType () const override ;
			nkGraphics::Texture* getSourceTexture () const ;
			nkGraphics::Sampler* getSourceSampler () const ;
			nkGraphics::Camera* getSourceCamera () const ;
			IMAGE_ENVIRONMENT_PROJECTION getSourceProjection () const ;

			// Setters
			void setSourceTexture (nkGraphics::Texture* value) ;
			void setSourceSampler (nkGraphics::Sampler* value) ;
			void setSourceCamera (nkGraphics::Camera* value) ;
			void setSourceProjection (IMAGE_ENVIRONMENT_PROJECTION value) ;

			// Load
			virtual bool load () override ;
			virtual void unload () override ;

			// Utilities
			static IMAGE_ENVIRONMENT_PROJECTION getRequiredProjection (const ImageEnvironmentEffectInfo& info) ;
	} ;
}