﻿//////////////////////////////////////////////
// Entity.h	
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Mesh ;
	class Node ;
	class RenderQueue ;
	class Shader ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Bounds/BoundingBox.h"

#include "EntityRenderInfo.h"

// nkExport
#include <NilkinsExport/Exportable/Exportable.h>

// Standards
#include <vector>

/// Class ------------------------------------

namespace nkGraphics
{	
	class DLL_GRAPHICS_EXPORT Entity final : public nkExport::Exportable
	{
		friend class RenderQueue ;

		private :
		
			// Attributes
			// Associated rendering info
			EntityRenderInfo _renderInfo ;

			// Parents
			Node* _parentNode ;
			RenderQueue* _renderQueue ;

		private :

			// Functions
			// Constructor, destructor
			Entity (RenderQueue* parentRenderQueue) ;
			~Entity () ;
		
		public :
		
			// Getters
			const EntityRenderInfo& getRenderInfo () const ;
			EntityRenderInfo& getRenderInfo () ;
			Node* getParentNode () const ;
			RenderQueue* getRenderQueue () const ;

			// Setters
			void setRenderInfo (const EntityRenderInfo& info) ;
			void setParentNode (Node* parent) ;
			void setDirtyNodeTransform (bool value) ;

			// Updates
			void updateMeshBinding (Mesh* oldMesh) ;			

			// Import / Export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}