﻿//////////////////////////////////////////////
// Program.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward declarations ---------------------

namespace nkGraphics
{
	class ProgramSourcesHolder ;
	class StreamOutputDescriptor ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Resources/Resource.h"

#include "../Utils/DxDefinesWrapper.h"

#include "ProgramCompilationResult.h"
#include "ProgramType.h"

// Export
#include <NilkinsExport/Exportable/Exportable.h>

// nkMemory
#include <NilkinsMemory/Containers/BufferView.h>
#include <NilkinsMemory/Containers/String.h>
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <vector>

/// Internals --------------------------------

namespace nkGraphics
{
	struct ProgramSemanticEntry
	{
		nkMemory::String _name ;
		unsigned int _index ;
	} ;
}

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT Program : public Resource, public nkExport::Exportable
	{
		protected :

			// Attributes
			// Last compilation result
			ProgramCompilationResult _lastCompilationResult ;

			// Memory, sources or extension to seek
			nkMemory::String _fileExtensions ;
			ProgramSourcesHolder* _memory ;

			// Semantics that the shader requires
			std::vector<ProgramSemanticEntry> _inputSemantics ;

			// Id for sorting
			unsigned int _id ;

			// Type of program inside
			PROGRAM_TYPE _type ;
			// Compilation profile
			PROGRAM_COMPILATION_PROFILE _profile ;

			// Flags
			bool _fromFile ;
			bool _filesLoaded ;
			bool _fromMemory ;
			bool _activateCrossCompilerDefines ;
		
		public :
		
			// Constructeur, destructeur
			Program () ;
			virtual ~Program () ;

			// Getters
			const ProgramCompilationResult& getLastCompilationResult () const ;
			bool isFromFile () const ;
			bool isFromMemory () const ;
			ProgramSourcesHolder* getMemory () const ;
			nkMemory::BufferView<const ProgramSemanticEntry> getInputSemantics () const ;
			unsigned int getId () const ;
			PROGRAM_TYPE getType () const ;
			PROGRAM_COMPILATION_PROFILE getProfile () const ;

			// Setters
			virtual void setFromMemory (const ProgramSourcesHolder& memory) ;
			virtual void setDefines (const ProgramSourcesHolder& memory) ;
			void setFromFiles (const nkMemory::StringView& folder) ;
			void setFileExtensions (const nkMemory::StringView& value) ;
			void setId (unsigned int id) ;
			void setProfile (PROGRAM_COMPILATION_PROFILE profile) ;
			void setActivateCrossCompilerDefines (bool value) ;

			// Fonctions de chargement
			bool prepareMemoryWithFiles () ;
			virtual bool loadFromMemory () = 0 ;
			virtual bool tryToLoadFrom (ProgramSourcesHolder& newMemory) = 0 ;

			// Load brut
			virtual bool load () override ;
			virtual void unload () override ;			

			// Import / Export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}