﻿//////////////////////////////////////////////
// FontManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkWinUi
{
	class Font ;
	class MainSystem ;
}

/// Includes ---------------------------------

// nkWinUi
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/MaybeSingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkWinUi
{
	class DLL_WINUI_EXPORT FontManager final : public nkCommon::MaybeSingletonClass<FontManager>
	{
		friend class nkCommon::MaybeSingletonClass<FontManager> ;
		friend class MainSystem ;

		private :

			// Attributes
			// Memory
			std::unordered_map<std::string, Font*> _memory ;

			MainSystem* _parentSystem ;
			Font* _defaultFont ;

		private :

			// Functions
			// Constructor, destructor
			FontManager () ;
			FontManager (MainSystem* parentSystem) ;
			~FontManager () ;
		
		public :

			// Getters
			Font* getDefaultFont () const ;
		
			// Memory management
			Font* createOrRetrieve (nkMemory::StringView name) ;
			Font* get (nkMemory::StringView name) const ;
			Font* getByIndex (unsigned int index) const ;
			void rename (nkMemory::StringView oldName, nkMemory::StringView newName) ;
			void erase (nkMemory::StringView name) ;
	} ;
}